@@ -22,19 +22,20 @@ import inspect
 import io
 import random as rnd_module
 import sys
+import time
 import traceback
+import types
 import warnings
 import zlib
 from inspect import getfullargspec
 from io import StringIO
 from random import Random
-from time import perf_counter
 from typing import Any, BinaryIO, Callable, Hashable, List, Optional, TypeVar, Union
 from unittest import TestCase
 
 import attr
 
-import hypothesis.strategies as st
+from hypothesis import strategies as st
 from hypothesis._settings import (
     HealthCheck,
     Phase,
@@ -353,12 +354,33 @@ def execute_explicit_examples(state, wrapped_test, arguments, kwargs):
                 # each of the (report text, error) pairs we find back to the top-level
                 # runner.  This also ensures that user-facing stack traces have as few
                 # frames of Hypothesis internals as possible.
-                yield (fragments_reported, err.with_traceback(get_trimmed_traceback()))
+                err = err.with_traceback(get_trimmed_traceback())
+
+                # One user error - whether misunderstanding or typo - we've seen a few
+                # times is to pass strategies to @example() where values are expected.
+                # Checking is easy, and false-positives not much of a problem, so:
+                if any(
+                    isinstance(arg, SearchStrategy)
+                    for arg in example.args + tuple(example.kwargs.values())
+                ):
+                    new = HypothesisWarning(
+                        "The @example() decorator expects to be passed values, but "
+                        "you passed strategies instead.  See https://hypothesis."
+                        "readthedocs.io/en/latest/reproducing.html for details."
+                    )
+                    new.__cause__ = err
+                    err = new
+
+                yield (fragments_reported, err)
                 if state.settings.report_multiple_bugs:
                     continue
                 break
+            finally:
+                assert fragments_reported[0].startswith("Falsifying example")
+                fragments_reported[0] = fragments_reported[0].replace(
+                    "Falsifying example", "Falsifying explicit example", 1
+                )
 
-            assert fragments_reported[0].startswith("Falsifying example")
             verbose_report(fragments_reported[0].replace("Falsifying", "Trying", 1))
             for f in fragments_reported[1:]:
                 verbose_report(f)
@@ -380,9 +402,7 @@ def get_random_for_wrapped_test(test, wrapped_test):
         return Random(seed)
 
 
-def process_arguments_to_given(
-    wrapped_test, arguments, kwargs, given_kwargs, argspec, settings,
-):
+def process_arguments_to_given(wrapped_test, arguments, kwargs, given_kwargs, argspec):
     selfy = None
     arguments, kwargs = convert_positional_arguments(wrapped_test, arguments, kwargs)
 
@@ -474,7 +494,7 @@ def new_given_argspec(original_argspec, given_kwargs):
 
 class StateForActualGivenExecution:
     def __init__(
-        self, test_runner, search_strategy, test, settings, random, wrapped_test,
+        self, test_runner, search_strategy, test, settings, random, wrapped_test
     ):
         self.test_runner = test_runner
         self.search_strategy = search_strategy
@@ -524,9 +544,9 @@ class StateForActualGivenExecution:
             def test(*args, **kwargs):
                 self.__test_runtime = None
                 initial_draws = len(data.draw_times)
-                start = perf_counter()
+                start = time.perf_counter()
                 result = self.test(*args, **kwargs)
-                finish = perf_counter()
+                finish = time.perf_counter()
                 internal_draw_time = sum(data.draw_times[initial_draws:])
                 runtime = datetime.timedelta(
                     seconds=finish - start - internal_draw_time
@@ -886,6 +906,7 @@ class HypothesisHandle:
 
     inner_test = attr.ib()
     _get_fuzz_target = attr.ib()
+    _given_kwargs = attr.ib()
 
     @property
     def fuzz_one_input(
@@ -911,7 +932,7 @@ class HypothesisHandle:
 
 def given(
     *_given_arguments: Union[SearchStrategy, InferType],
-    **_given_kwargs: Union[SearchStrategy, InferType]
+    **_given_kwargs: Union[SearchStrategy, InferType],
 ) -> Callable[[Callable[..., None]], Callable[..., None]]:
     """A decorator for turning a test function that accepts arguments into a
     randomized test.
@@ -998,7 +1019,7 @@ def given(
             random = get_random_for_wrapped_test(test, wrapped_test)
 
             processed_args = process_arguments_to_given(
-                wrapped_test, arguments, kwargs, given_kwargs, argspec, settings,
+                wrapped_test, arguments, kwargs, given_kwargs, argspec
             )
             arguments, kwargs, test_runner, search_strategy = processed_args
 
@@ -1021,7 +1042,7 @@ def given(
                 )
 
             state = StateForActualGivenExecution(
-                test_runner, search_strategy, test, settings, random, wrapped_test,
+                test_runner, search_strategy, test, settings, random, wrapped_test
             )
 
             reproduce_failure = wrapped_test._hypothesis_internal_use_reproduce_failure
@@ -1102,7 +1123,7 @@ def given(
                 if isinstance(runner, TestCase) and hasattr(runner, "subTest"):
                     subTest = runner.subTest
                     try:
-                        runner.subTest = fake_subTest
+                        runner.subTest = types.MethodType(fake_subTest, runner)
                         state.run_engine()
                     finally:
                         runner.subTest = subTest
@@ -1158,14 +1179,19 @@ def given(
             )
             random = get_random_for_wrapped_test(test, wrapped_test)
             _args, _kwargs, test_runner, search_strategy = process_arguments_to_given(
-                wrapped_test, (), {}, given_kwargs, argspec, settings,
+                wrapped_test, (), {}, given_kwargs, argspec
             )
             assert not _args
             assert not _kwargs
             state = StateForActualGivenExecution(
-                test_runner, search_strategy, test, settings, random, wrapped_test,
+                test_runner, search_strategy, test, settings, random, wrapped_test
             )
             digest = function_digest(test)
+            # We track the minimal-so-far example for each distinct origin, so
+            # that we track log-n instead of n examples for long runs.  In particular
+            # it means that we saturate for common errors in long runs instead of
+            # storing huge volumes of low-value data.
+            minimal_failures: dict = {}
 
             def fuzz_one_input(
                 buffer: Union[bytes, bytearray, memoryview, BinaryIO]
@@ -1181,8 +1207,13 @@ def given(
                 except (StopTest, UnsatisfiedAssumption):
                     return None
                 except BaseException:
-                    if settings.database is not None:
-                        settings.database.save(digest, bytes(data.buffer))
+                    buffer = bytes(data.buffer)
+                    known = minimal_failures.get(data.interesting_origin)
+                    if settings.database is not None and (
+                        known is None or sort_key(buffer) <= sort_key(known)
+                    ):
+                        settings.database.save(digest, buffer)
+                        minimal_failures[data.interesting_origin] = buffer
                     raise
                 return bytes(data.buffer)
 
@@ -1208,7 +1239,7 @@ def given(
         wrapped_test._hypothesis_internal_use_reproduce_failure = getattr(
             test, "_hypothesis_internal_use_reproduce_failure", None
         )
-        wrapped_test.hypothesis = HypothesisHandle(test, _get_fuzz_target)
+        wrapped_test.hypothesis = HypothesisHandle(test, _get_fuzz_target, given_kwargs)
         return wrapped_test
 
     return run_test_as_given
@@ -1219,9 +1250,9 @@ def find(
     specifier: SearchStrategy[Ex],
     condition: Callable[[Any], bool],
     *,
-    settings: Settings = None,
-    random: Random = None,
-    database_key: bytes = None
+    settings: Optional[Settings] = None,
+    random: Optional[Random] = None,
+    database_key: Optional[bytes] = None,
 ) -> Ex:
     """Returns the minimal example from the given strategy ``specifier`` that
     matches the predicate function ``condition``."""
